@@ -31,7 +31,7 @@ knearneigh <- function(x, k=1, longlat=NULL, RANN=TRUE)
     zd <- zerodist(SpatialPoints(x))
     if (!(nrow(zd) == 0)) warning("knearneigh: identical points found")
     if (RANN && !longlat && (nrow(zd) == 0) && 
-        (requireNamespace("spam", quietly = TRUE)) ) {
+        (requireNamespace("RANN", quietly = TRUE)) ) {
 #        xx <- cbind(x, out=rep(0, nrow(x)))
 #        out <- as.matrix(nn(xx, p=k)$nn.idx)
 # nn() retired 130722
