@@ -46,21 +46,20 @@ fitted.sarlm <- function(object, ...) {
 }
 
 
-# retourne la valeur de la prédiction + un attributs trend et signal
-predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=FALSE,
+predict.sarlm <- function(object, newdata=NULL, listw=NULL, pred.type="TS", all.data=FALSE,
                           zero.policy=NULL, legacy=TRUE, legacy.mixed=FALSE, power=NULL, order=250, tol=.Machine$double.eps^(3/5), #pred.se=FALSE, lagImpact=NULL, 
                           spChk=NULL, ...) {
   if (is.null(zero.policy))
     zero.policy <- get("zeroPolicy", envir = .spdepOptions)
   stopifnot(is.logical(zero.policy))
-  if (is.null(type)) type <- "default"
-  # check type with model
-  if (type %in% c("default", "TS") & object$type %in% c("sac", "sacmixed")) stop("no such predict method for sac model")
-  if (type %in% c("TC", "TS", "BP", "BPW", "BPN", "TS1") & object$type == "error") stop("no such predict method for error model")
-  if (type %in% c("KP5") & object$type %in% c("lag", "lagmixed")) stop("no such predict method for lag model")
+  if (is.null(pred.type)) pred.type <- "TS"
+  # check pred.type with model
+  if (pred.type %in% c("TS") & object$type %in% c("sac", "sacmixed")) stop("no such predict method for sac model")
+  if (pred.type %in% c("TC", "BP", "BPW", "BPN", "TS1", "BP1", "BPW1", "BPN1") & object$type == "error") stop("no such predict method for error model")
+  if (pred.type %in% c("KP5") & object$type %in% c("lag", "lagmixed")) stop("no such predict method for lag model")
   
-  if (type %in% c("BP", "BPW", "BPN") & object$type %in% c("sac", "sacmixed")) warning("predict method developed for lag model, use carefully")
-  if (type %in% c("KP5") & object$type %in% c("sac", "sacmixed")) warning("predict method developed for sem model, use carefully")
+  if (pred.type %in% c("TC", "BP", "BPW", "BPN", "BP1", "BPW1", "BPN1") & object$type %in% c("sac", "sacmixed")) warning("predict method developed for lag model, use carefully")
+  if (pred.type %in% c("KP5") & object$type %in% c("sac", "sacmixed")) warning("predict method developed for sem model, use carefully")
   
   
   if (is.null(power)) power <- object$method != "eigen"
@@ -89,7 +88,8 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
   # forecast case: newdata with the same names than data
   # use a sub-samble of in-sample predictors
   if (!is.null(newdata) && nrow(newdata) == length(ys) && rownames(newdata) == attr(ys, "names")) {
-    if (!type %in% c("trend", "TC")) warning("no such predictor type for prevision")
+    if (!pred.type %in% c("trend", "TC", "TS"))
+      warning("no such predictor type for prevision")
     # DATA
     frm <- formula(object$call)
     mt <- delete.response(terms(frm, data=newdata)) # returns a terms object for the same model but with no response variable
@@ -99,7 +99,7 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
     if (dim(mf)[1] != nrow(newdata))
       stop("missing values in newdata")
     Xs <- model.matrix(mt, mf)
-        
+    
     if (object$type == "mixed" || (object$type == "error" && object$etype == "emixed")) { # mixed model: compute WXo
       if (is.null(listw) || !inherits(listw, "listw"))
         stop ("spatial weights list required")
@@ -107,6 +107,11 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
         stop("mismatch between data and spatial weights")
       if (spChk && !chkIDs(Xs, listw))
         stop("Check of data and weights ID integrity failed")
+      if (pred.type == "TS" && legacy.mixed == FALSE) { # force legacy.mixed to allow the TS out-of-sample computation in the prevision case 
+        warning("only legacy.mixed=TRUE is supported for pred.type='TS' and mixed models. legacy.mixed is forced")
+        legacy.mixed <- TRUE
+      }
+      
       K <- ifelse(colnames(Xs)[1] == "(Intercept)", 2, 1)
       m <- ncol(Xs)
       # check if there are enough regressors
@@ -132,7 +137,11 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
             WXs <- matrix(wx, nrow = nrow(Xs), ncol = 1)
           }
         } 
-      }   
+      }
+      if (any(object$aliased)) {
+        if (K>1 && (listw$style == "W")) colnames(WXs) <- paste("lag.", colnames(Xs)[-1], sep="")
+        else colnames(WXs) <- paste("lag.", colnames(Xs), sep="")
+      }
       Xs <- cbind(Xs, WXs)
     }
     #  accommodate aliased coefficients 120314
@@ -141,12 +150,13 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
     
     tarXs <- tarys <- NULL
     trends <- Xs %*% B
-    newdata <- NULL
+    if (pred.type != "TS") # use the out-of-sample computation of TS (which only depends on Woo)
+      newdata <- NULL
   }
   
   
   if (is.null(newdata)) { # in-sample pred
-    if (type == "default" || type == "TS") { # defaut predictor
+    if (pred.type == "TS") { # defaut predictor
       res <- fitted.values(object)
       if (object$type == "error") { # We ou WX+We
         attr(res, "trend") <- as.vector(trends)
@@ -156,7 +166,7 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
         attr(res, "signal") <- as.vector( -1 * (tarys - ys))
       }
     } else { # new predictors
-      if (type != "trend") { # need listw
+      if (pred.type != "trend") { # need listw
         if (is.null(listw) || !inherits(listw, "listw"))
           stop ("spatial weights list required")
         if (nrow(Xs) != length(listw$neighbours))
@@ -165,7 +175,7 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
           stop("Check of data and weights ID integrity failed")
       }
       
-      if (type %in% c("TC", "BP")) { # need to compute TC
+      if (pred.type %in% c("TC", "BP")) { # need to compute TC
         if (power){
           W <- as(listw, "CsparseMatrix")
           TC <- powerWeights(W, rho = object$rho, X = Xs, order = order, tol = tol) %*% B
@@ -174,11 +184,11 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
         }
       }
       
-      if (type == "trend") {
+      if (pred.type == "trend") {
         res <- as.vector(trends)
-      } else if(type == "TC") {
+      } else if(pred.type == "TC") {
         res <- as.vector(TC)
-      } else if(type == "BP") {
+      } else if(pred.type == "BP") {
         W <- as(listw, "CsparseMatrix")
         Qss <- 1/object$s2 * (Diagonal(dim(W)[1]) - (object$rho * t(W))) %*% (Diagonal(dim(W)[1]) - (object$rho * W)) # precision matrix for LAG model
         DiagQss <- Diagonal(x = diag(Qss))
@@ -188,13 +198,14 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
       } else {
         stop("no such in-sample predictor type")
       }
-      if (type != "trend") attr(res, "trend") <- as.vector(trends)
+      if (pred.type != "trend") attr(res, "trend") <- as.vector(trends)
     }
     attr(res, "region.id") <- as.vector(attr(ys, "names"))
   } else { # out-of-sample
     #CHECK
-    if (any(rownames(newdata) %in% attr(ys, "names"))) warning("some region.id are both in data and newdata")
-    if (!(type == "default" && object$type == "error" && object$etype == "error") && !(type == "trend" && (object$type != "mixed" && !(object$type == "error" && object$etype == "emixed")))) { # need of listw (ie. neither in the case of defaut predictor and SEM model, nor trend type without mixed models)
+    if (any(rownames(newdata) %in% attr(ys, "names")) && !(pred.type == "TS" && nrow(newdata) == length(ys) && rownames(newdata) == attr(ys, "names"))) # no warning in the computation of TS in forecast
+      warning("some region.id are both in data and newdata")
+    if (!(pred.type == "TS" && object$type == "error" && object$etype == "error") && !(pred.type == "trend" && (object$type != "mixed" && !(object$type == "error" && object$etype == "emixed")))) { # need of listw (ie. neither in the case of defaut predictor and SEM model, nor trend type without mixed models)
       if (is.null(listw) || !inherits(listw, "listw"))
         stop ("spatial weights list required")
       if (any(! rownames(newdata) %in% attr(listw, "region.id")))
@@ -202,17 +213,20 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
       listw.old <- NULL
       
       # wanted order of the listw
-      if (type == "default") { # only need Woo
+      if (pred.type == "TS") { # only need Woo
         region.id <- rownames(newdata)
         if (!legacy.mixed) listw.old <- listw # keep the old listw to allow the computation of lagged variable from the full WX
+        # listw <- mat2listw(matrix(0), row.names = rownames(newdata)) # avoid crash of subset.listw
       } else {
         region.id <- c(attr(ys, "names"), rownames(newdata))
+        if (legacy.mixed) listw.old <- listw # keep the old listw to allow the computation of lagged variable from Woo (as mat2listw() cannot compute Woo from the new listw, because it has general weights lists)
       }
       
       if (length(region.id) != length(attr(listw, "region.id")) || !all(region.id == attr(listw, "region.id"))) { # if listw is not directly ok
-        if (all(subset(attr(listw, "region.id"), attr(listw, "region.id") %in% region.id) == region.id)) { # only need a subset.listw, ie. spatial units are in the right order
+        if (all(subset(attr(listw, "region.id"), attr(listw, "region.id") %in% region.id) == region.id) && listw$style != "M") { # only need a subset.listw, ie. spatial units are in the right order and if weights style is not unknown
           listw <- subset.listw(listw, (attr(listw, "region.id") %in% region.id), zero.policy = zero.policy)
         } else { # we use a sparse matrix transformation to reorder a listw
+          if (listw$style == "M") warning("unknown weight style: a subset of listw is used without re-normalization")
           W <- as(listw, "CsparseMatrix")
           W <- W[region.id, region.id]
           style <- listw$style
@@ -257,11 +271,15 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
         region.id.mixed <- rownames(X)
       }
       if (is.null(listw.old)) listw.mixed <- listw
-      else listw.mixed <- listw.old
+      else {
+        listw.mixed <- listw.old
+        rm(listw.old)
+      }
       if (length(region.id.mixed) != length(attr(listw.mixed, "region.id")) || !all(region.id.mixed == attr(listw.mixed, "region.id"))) { # if listw is not directly ok
-        if (all(subset(attr(listw.mixed, "region.id"), attr(listw.mixed, "region.id") %in% region.id.mixed) == region.id.mixed)) { # only need a subset.listw, ie. spatial units are in the right order
+        if (all(subset(attr(listw.mixed, "region.id"), attr(listw.mixed, "region.id") %in% region.id.mixed) == region.id.mixed) && listw.mixed$style != "M") { # only need a subset.listw, ie. spatial units are in the right order and weights style is not unknown
           listw.mixed <- subset.listw(listw.mixed, attr(listw.mixed, "region.id") %in% region.id.mixed, zero.policy = zero.policy)
         } else { # we use a sparse matrix transformation to reorder a listw
+          if (listw$style == "M") warning("unknown weight style: a subset of listw is used without re-normalization")
           W <- as(listw.mixed, "CsparseMatrix")
           W <- W[region.id.mixed, region.id.mixed]
           style <- listw.mixed$style
@@ -302,7 +320,7 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
         } 
       }
       if (any(object$aliased)) {
-        if (K>1) colnames(WX) <- paste("lag.", colnames(X)[-1], sep="")
+        if (K>1 && (listw.mixed$style == "W")) colnames(WX) <- paste("lag.", colnames(X)[-1], sep="")
         else colnames(WX) <- paste("lag.", colnames(X), sep="")
         WX <- WX[,!colnames(WX) %in% names(object$aliased[object$aliased])]
       }
@@ -317,7 +335,7 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
     }
     trendo <- Xo %*% B
     
-    if (type == "default") { # defaut predictor
+    if (pred.type == "TS") { # defaut predictor
       if (object$type == "error") {
         if (object$etype == "error") { # We
           signal <- rep(0, length(trendo))
@@ -326,7 +344,7 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
           attr(res, "signal") <- signal
         } else if (object$etype == "emixed") { # WX + We
           signal <- rep(0, length(trendo))
-          res <- trend + signal
+          res <- trendo + signal
           attr(res, "trend") <- trendo
           attr(res, "signal") <- signal
         } else stop("unkown error model etype")
@@ -335,7 +353,7 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
           W <- as(listw, "CsparseMatrix")
           res <- c(as(powerWeights(W, rho=object$rho,
                                    X=trendo, order=order, tol=tol), "matrix"))
-        } else { # calcul de (I - rho*W)^-1 en inversant la matrice
+        } else {
           res <- c(invIrW(listw, object$rho) %*% trendo)
         }
         if (legacy) {
@@ -408,14 +426,14 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
         attr(res, "signal") <- c(signal)
       }
     } else { # new predictors
-      if (type %in% c("TS1", "KP4", "KP2", "KP3")) { # need to compute TS1/KP4
+      if (pred.type %in% c("TS1", "KP4", "KP2", "KP3")) { # need to compute TS1/KP4
         Wos <- .listw.decompose(listw, region.id.data = attr(ys, "names"), region.id.newdata = rownames(newdata), type = "Wos")$Wos
         if (is.null(object$rho)) TS1 <- Xo %*% B
         else TS1 <- Xo %*% B + object$rho * Wos %*% ys
       }
-      if (type %in% c("TC", "BP", "BPN")) { # need to compute TC
+      if (pred.type %in% c("TC", "BP", "BPN")) { # need to compute TC
         #notations of C.Thomas and al (2015)
-        if (all.data | type %in% c("BP", "BPN")) {
+        if (all.data | pred.type %in% c("BP", "BPN")) {
           # compute s and o units together
           X <- rbind(Xs, Xo)
           trend <- X %*% B
@@ -450,17 +468,17 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
         }
       }
       
-      if (type == "trend") {
+      if (pred.type == "trend") {
         res <- as.vector(trendo)
-      } else if (type %in% c("TS1", "KP4")) {
+      } else if (pred.type %in% c("TS1", "KP4")) {
         res <- as.vector(TS1)
-      } else if (type == "TC") {
+      } else if (pred.type == "TC") {
         if(all.data) {
           res <- as.vector(TC)
         } else {
           res <- as.vector(TCo)
         }
-      } else if (type == "BP") {
+      } else if (pred.type == "BP") {
         is.data <- 1:length(ys)
         is.newdata <- (length(ys)+1):length(TC)
         TCo <- TC[is.newdata]
@@ -471,7 +489,7 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
         Qos <- Q[is.newdata, is.data]
         BPo <- TCo - solve(Qoo) %*% Qos %*% (ys - TCs)
         res <- as.vector(BPo)
-      } else if (type == "BPW") {
+      } else if (pred.type == "BPW") {
         if (power){
           W <- as(listw, "CsparseMatrix")
           invW <- powerWeights(W, rho = object$rho, X = Diagonal(dim(W)[1]), order = order, tol = tol)
@@ -486,12 +504,12 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
         TCs <- TC[is.data]
         #Sigma <- object$s2 * solve((Diagonal(dim(W)[1]) - object$rho * t(W)) %*% (Diagonal(dim(W)[1]) - object$rho * W))
         Sigma <- object$s2 * invW %*% t(invW)
-        Sos <- Sigma[is.newdata, is.data]
-        Sss <- Sigma[is.data, is.data]
+        Sos <- Sigma[is.newdata, is.data, drop=F]
+        Sss <- Sigma[is.data, is.data, drop=F]
         Wos <- .listw.decompose(listw, region.id.data = attr(ys, "names"), region.id.newdata = rownames(newdata), type = "Wos")$Wos
-        BPW <- TCo + Sos %*% t(Wos) %*% solve(Wos %*% Sss %*% t(Wos)) %*% (Wos %*% ys - Wos %*% TCs)
+        BPW <- as.numeric(TCo + Sos %*% t(Wos) %*% solve(Wos %*% Sss %*% t(Wos)) %*% (Wos %*% ys - Wos %*% TCs))
         res <- as.vector(BPW)
-      } else if (type == "BPN") {
+      } else if (pred.type == "BPN") {
         is.data <- 1:length(ys)
         is.newdata <- (length(ys)+1):length(TC)
         TCs <- TC[is.data]
@@ -499,12 +517,19 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
         # compute J = set of all sites in S which are neighbors of at least one site in O
         O <- which(attr(listw,"region.id") %in% rownames(newdata))
         S <- which(attr(listw,"region.id") %in% attr(ys, "names"))
-        J.logical <- rep(FALSE, length(listw$neighbours))
-        for (i in S) {
-          J.logical[i] <- any(O %in% listw$neighbours[[i]])
+        #bug fix: J = set of S s.t. at least 1 elmt of O is neigbourg with S. And not: set of S s.t. these elmts of S are neigbourgs with at least 1 elmt of S.
+        #different iff W is not symetric
+        #J.logical <- rep(FALSE, length(listw$neighbours))
+        #for (i in S) {
+        #  J.logical[i] <- any(O %in% listw$neighbours[[i]])
+        #}
+        #J <- attr(listw,"region.id")[J.logical]
+        J.ref <- NULL
+        for (i in O) {
+          J.ref <- c(J.ref, listw$neighbours[[i]][listw$neighbours[[i]] %in% S])
         }
-        J <- attr(listw,"region.id")[J.logical]
-        
+        J <- attr(listw,"region.id")[unique(J.ref)]
+        J <- attr(listw,"region.id")[attr(listw,"region.id") %in% J] # keep the order of listw
         if (length(J)<1) {
           warning("out-of-sample units have no neighbours")
           BPN <- TCo
@@ -524,26 +549,155 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
           BPN <- as.vector(TCo - solve(Qoo) %*% Qoj %*% (yj - TCj))
         }
         res <- as.vector(BPN)
-      } else if (type %in% c("TC1", "KP1")) {
-        #TODO: use the definition of TCo of C.Thomas applied in the leave-one-out case: quicker?
+      } else if (pred.type %in% c("TC1", "KP1")) {
         if (nrow(newdata) > 1)
           warning("newdata have more than 1 row and the predictor type is leave-one-out")
         region.id.data <- attr(ys, "names")
         region.id.newdata <- rownames(newdata)
         res <- rep(NA, nrow(newdata))
         W <- as(listw, "CsparseMatrix")
+        style <- listw$style
+        if (listw$style == "M")
+          warning("unknown weight style: a subset of listw is used without re-normalization")
         for (i in 1:nrow(newdata)) {
           region.id.temp <- c(region.id.data, region.id.newdata[i])
           Wi <- W[region.id.temp, region.id.temp]
+          listwi <- mat2listw(Wi, row.names = region.id.temp, style = style) # re-normalize
+          if (power)
+            Wi <- as(listwi, "CsparseMatrix")
           Xi <- rbind(Xs, Xo[i,])
           if (power) {
             res[i] <- c(as(powerWeights(Wi, rho=object$rho, X=Xi, order=order, tol=tol), "matrix") %*% B)[length(region.id.temp)]
           } else {
             trendi <- c(trends, trendo[i])
-            res[i] <- (invIrW(Wi, object$rho) %*% trendi)[length(region.id.temp)]
+            res[i] <- (invIrW(listwi, object$rho) %*% trendi)[length(region.id.temp)]
           }
         }
-      } else if (type == "KP2") {
+      } else if (pred.type == "BP1") {
+        if (nrow(newdata) > 1)
+          warning("newdata have more than 1 row and the predictor type is leave-one-out")
+        region.id.data <- attr(ys, "names")
+        region.id.newdata <- rownames(newdata)
+        BP1o <- rep(NA, nrow(newdata))
+        W <- as(listw, "CsparseMatrix")
+        style <- listw$style
+        if (listw$style == "M")
+          warning("unknown weight style: a subset of listw is used without re-normalization")
+        for (i in 1:nrow(newdata)) {
+          region.id.temp <- c(region.id.data, region.id.newdata[i])
+          Wi <- W[region.id.temp, region.id.temp]
+          listwi <- mat2listw(Wi, row.names = region.id.temp, style = style) # re-normalize
+          Wi <- as(listwi, "CsparseMatrix")
+          Xi <- rbind(Xs, Xo[i,])
+          # compute TC1 for S and o units
+          TC1i <- rep(NA, length(region.id.temp))
+          is.data <- 1:length(ys)
+          is.newdata <- length(region.id.temp)
+          if (power) {
+            TC1i <- c(as(powerWeights(Wi, rho=object$rho, X=Xi, order=order, tol=tol), "matrix") %*% B)
+          } else {
+            trendi <- c(trends, trendo[i])
+            TC1i <- (invIrW(Wi, object$rho) %*% trendi)
+          }
+          TC1si <- TC1i[is.data]
+          TC1oi <- TC1i[is.newdata]
+          
+          Qi <- 1/object$s2 * ( Diagonal(dim(Wi)[1]) - object$rho * (t(Wi) + Wi) + object$rho^2 * (t(Wi) %*% Wi) )
+          Qooi <- Qi[is.newdata, is.newdata]
+          Qosi <- Qi[is.newdata, is.data]
+          BP1o[i] <- TC1oi - solve(Qooi) %*% Qosi %*% (ys - TC1si)
+        }
+        res <- as.vector(BP1o)
+      } else if (pred.type == "BPW1") {
+        if (nrow(newdata) > 1)
+          warning("newdata have more than 1 row and the predictor type is leave-one-out")
+        region.id.data <- attr(ys, "names")
+        region.id.newdata <- rownames(newdata)
+        BPW1o <- rep(NA, nrow(newdata))
+        W <- as(listw, "CsparseMatrix")
+        style <- listw$style
+        if (listw$style == "M")
+          warning("unknown weight style: a subset of listw is used without re-normalization")
+        for (i in 1:nrow(newdata)) {
+          region.id.temp <- c(region.id.data, region.id.newdata[i])
+          Wi <- W[region.id.temp, region.id.temp]
+          listwi <- mat2listw(Wi, row.names = region.id.temp, style = style) # re-normalize
+          Wi <- as(listwi, "CsparseMatrix")
+          Xi <- rbind(Xs, Xo[i,])
+          is.data <- 1:length(ys)
+          is.newdata <- length(region.id.temp)
+          if (power){
+            invWi <- powerWeights(Wi, rho = object$rho, X = Diagonal(dim(Wi)[1]), order = order, tol = tol)
+          } else {
+            invWi <- invIrW(Wi, object$rho)
+          }
+          TC1i <- invWi %*% Xi %*% B
+          TC1oi <- TC1i[is.newdata]
+          TC1si <- TC1i[is.data]
+          #Sigma <- object$s2 * solve((Diagonal(dim(W)[1]) - object$rho * t(W)) %*% (Diagonal(dim(W)[1]) - object$rho * W))
+          Sigmai <- object$s2 * invWi %*% t(invWi)
+          Sosi <- Sigmai[is.newdata, is.data, drop=F]
+          Sssi <- Sigmai[is.data, is.data]
+          Wosi <- Wi[region.id.newdata[i], region.id.data, drop=F]
+          BPW1o[i] <- as.numeric(TC1oi + Sosi %*% t(Wosi) %*% solve(Wosi %*% Sssi %*% t(Wosi)) %*% (Wosi %*% ys - Wosi %*% TC1si))
+        }
+        res <- as.vector(BPW1o)
+      } else if (pred.type == "BPN1") {
+        if (nrow(newdata) > 1)
+          warning("newdata have more than 1 row and the predictor type is leave-one-out")
+        region.id.data <- attr(ys, "names")
+        region.id.newdata <- rownames(newdata)
+        BPN1o <- rep(NA, nrow(newdata))
+        W <- as(listw, "CsparseMatrix")
+        style <- listw$style
+        if (listw$style == "M")
+          warning("unknown weight style: a subset of listw is used without re-normalization")
+        for (i in 1:nrow(newdata)) {
+          region.id.temp <- c(region.id.data, region.id.newdata[i])
+          Wi <- W[region.id.temp, region.id.temp]
+          listwi <- mat2listw(Wi, row.names = region.id.temp, style = style) # re-normalize
+          Wi <- as(listwi, "CsparseMatrix")
+          Xi <- rbind(Xs, Xo[i,])
+          # compute TC1 for S and o units
+          TC1i <- rep(NA, length(region.id.temp))
+          is.data <- 1:length(ys)
+          is.newdata <- length(region.id.temp)
+          if (power) {
+            TC1i <- c(as(powerWeights(Wi, rho=object$rho, X=Xi, order=order, tol=tol), "matrix") %*% B)
+          } else {
+            trendi <- c(trends, trendo[i])
+            TC1i <- (invIrW(Wi, object$rho) %*% trendi)
+          }
+          TC1si <- TC1i[is.data]
+          TC1oi <- TC1i[is.newdata]
+          
+          # compute J = set of all sites in S which are neighbors of at least one site in O
+          o <- which(attr(listwi,"region.id") %in% rownames(newdata)[i])
+          S <- which(attr(listwi,"region.id") %in% attr(ys, "names"))
+          J.ref <- NULL
+          for (k in o) {
+            J.ref <- c(J.ref, listwi$neighbours[[k]][listwi$neighbours[[k]] %in% S])
+          }
+          J <- attr(listwi,"region.id")[unique(J.ref)]
+          J <- attr(listwi,"region.id")[attr(listwi,"region.id") %in% J] # keep the order of listw
+          if (length(J)<1) {
+            warning("out-of-sample units have no neighbours")
+            BPN1o[i] <- TC1oi
+          } else {
+            region.id <- c(J, rownames(newdata)[i])
+            W_jo <- Wi[region.id, region.id, drop=F]
+            Q_jo <- 1/object$s2 * (Diagonal(length(region.id)) - object$rho * (W_jo + t(W_jo)) + object$rho^2 * (t(W_jo) %*% W_jo))
+            is.j <- 1:length(J)
+            is.o <- (length(J)+1):length(region.id)
+            Qoo <- Q_jo[is.o, is.o, drop=F]
+            Qoj <- Q_jo[is.o, is.j, drop=F]
+            yj <- ys[J]
+            TC1j <- TC1si[attr(ys, "names") %in% J]
+            BPN1o[i] <- as.vector(TC1oi - solve(Qoo) %*% Qoj %*% (yj - TC1j))
+          }
+        }
+        res <- as.vector(BPN1o)
+      } else if (pred.type == "KP2") {
         if (nrow(newdata) > 1)
           warning("newdata have more than 1 row and the predictor type is leave-one-out")
         region.id.data <- attr(ys, "names")
@@ -590,7 +744,7 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
           KP2[i] <- TS1[i] + covar %*% (wi %*% yi - Ewiy)
         }
         res <- as.vector(KP2)
-      } else if (type == "KP3") {
+      } else if (pred.type == "KP3") {
         if (nrow(newdata) > 1)
           warning("newdata have more than 1 row and the predictor type is leave-one-out")
         region.id.data <- attr(ys, "names")
@@ -636,7 +790,7 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
           KP3[i] <- as.vector(TS1[i] + cov %*% solve(sum.y[-length(region.id.temp), -length(region.id.temp)]) %*% (ys - GR[-length(region.id.temp),] %*% Xi %*% B))
         }
         res <- as.vector(KP3)
-      } else if (type == "KP5") {
+      } else if (pred.type == "KP5") {
         if (nrow(newdata) > 1)
           warning("newdata have more than 1 row and the predictor type is leave-one-out")
         Wos <- .listw.decompose(listw, region.id.data = attr(ys, "names"), region.id.newdata = rownames(newdata), type = "Wos")$Wos
@@ -652,7 +806,7 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
       attr(res, "region.id") <- c(attr(ys, "names"), rownames(newdata))
     } else stop("incorrect final output")
   }
-  attr(res, "type") <- type
+  attr(res, "pred.type") <- pred.type
   attr(res, "call") <- match.call()
   class(res) <- "sarlm.pred"
   res
@@ -672,13 +826,13 @@ predict.sarlm <- function(object, newdata=NULL, listw=NULL, type=NULL, all.data=
   W <- as(listw, "CsparseMatrix")
   s <- list(Wss = NULL, Wos = NULL, Wso = NULL, Woo = NULL)
   if ("Wss" %in% type)
-    s$Wss <- W[region.id.data, region.id.data]
+    s$Wss <- W[region.id.data, region.id.data, drop=F]
   if ("Wos" %in% type)
-    s$Wos <- W[region.id.newdata, region.id.data]
+    s$Wos <- W[region.id.newdata, region.id.data, drop=F]
   if ("Wso" %in% type)
-    s$Wso <- W[region.id.data, region.id.newdata]
+    s$Wso <- W[region.id.data, region.id.newdata, drop=F]
   if ("Woo" %in% type)
-    s$Woo <- W[region.id.newdata, region.id.newdata]
+    s$Woo <- W[region.id.newdata, region.id.newdata, drop=F]
   return(s)
 }
 
